<?php

declare(strict_types=1);

namespace Drupal\modeler_api\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\modeler_api\Plugin\ModelerPluginManager;
use Drupal\modeler_api\Plugin\ModelOwnerPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Modeler API settings for this site.
 */
final class Settings extends ConfigFormBase {

  public const string STORAGE_OPTION_NONE = 'none';
  public const string STORAGE_OPTION_THIRD_PARTY = 'third-party';
  public const string STORAGE_OPTION_SEPARATE = 'separate';

  /**
   * The model owner plugin manager.
   *
   * @var \Drupal\modeler_api\Plugin\ModelOwnerPluginManager
   */
  protected ModelOwnerPluginManager $ownerPluginManager;

  /**
   * The modeler plugin manager.
   *
   * @var \Drupal\modeler_api\Plugin\ModelerPluginManager
   */
  protected ModelerPluginManager $modelerPluginManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    $instance = parent::create($container);
    $instance->ownerPluginManager = $container->get('plugin.manager.modeler_api.model_owner');
    $instance->modelerPluginManager = $container->get('plugin.manager.modeler_api.modeler');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'modeler_api_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['modeler_api.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $storageOptions = [
      self::STORAGE_OPTION_NONE => $this->t('Do not store raw model data'),
      self::STORAGE_OPTION_SEPARATE => $this->t('Store raw data in separate config entity'),
      self::STORAGE_OPTION_THIRD_PARTY => $this->t('Store raw data with config as third-party setting'),
    ];
    $form['storage'] = [
      '#type' => 'details',
      '#title' => $this->t('Storage'),
      '#tree' => TRUE,
      '#open' => TRUE,
    ];
    foreach ($this->ownerPluginManager->getAllInstances() as $owner) {
      $form['storage'][$owner->getPluginId()] = [
        '#type' => 'details',
        '#title' => $this->t(':label', [':label' => $owner->label()]),
        '#open' => TRUE,
      ];
      foreach ($this->modelerPluginManager->getAllInstances() as $modeler) {
        if ($modeler->getPluginId() === 'fallback') {
          continue;
        }
        $form['storage'][$owner->getPluginId()][$modeler->getPluginId()] = [
          '#type' => 'select',
          '#title' => $this->t(':label', [':label' => $modeler->label()]),
          '#default_value' => $this->config('modeler_api.settings')->get('storage.' . $owner->getPluginId() . '.' . $modeler->getPluginId()) ?? self::STORAGE_OPTION_THIRD_PARTY,
          '#options' => $storageOptions,
        ];
      }
    }
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $config = $this->config('modeler_api.settings');
    foreach ($this->ownerPluginManager->getAllInstances() as $owner) {
      foreach ($this->modelerPluginManager->getAllInstances() as $modeler) {
        if ($modeler->getPluginId() === 'fallback') {
          continue;
        }
        $config->set('storage.' . $owner->getPluginId() . '.' . $modeler->getPluginId(), $form_state->getValue([
          'storage',
          $owner->getPluginId(),
          $modeler->getPluginId(),
        ]));
      }
    }
    $config->save();
    parent::submitForm($form, $form_state);
  }

}
